#!/usr/bin/python
# Written by Ara Pulido <ara@ubuntu.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


# Set database settings
# Database settings
#db_host="/var/run/postgresql/"
#db_user=
#db_password=
#db_database

import sys, pgdb, re, os, operator
from optparse import OptionParser

# Connect to the DB
try:
    db=pgdb.connect(user=db_user,password=db_password,host=db_host,database=db_database)
    cursor=db.cursor()
except:
    sys.exit("Failed to connect to the DB")


def getImagesForMilestone(milestone):
   
    sql = "SELECT qatracker_build.id, qatracker_build.version, \
                qatracker_build.date, qatracker_build.status as buildstatus, \
                qatracker_milestone.title as milestone, qatracker_milestone.status, \
                qatracker_product.id as productid, qatracker_product.title as product, qatracker_product.type, \
                qatracker_product.icon \
                FROM qatracker_build \
                LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid \
                LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid \
                WHERE qatracker_milestone.id='"+str(milestone)+"' AND (qatracker_build.status='0' OR qatracker_build.status='1') AND qatracker_product.siteid='1' \
                ORDER BY qatracker_product.title ASC"

    cursor.execute(sql)
    images = cursor.fetchall()

    return images

def getTestcases(build, type="all"):

    if type == "mandatory":
        sql = "SELECT qatracker_testcase.id, \
                  qatracker_testcase.title, \
                  qatracker_testcase.link \
                  FROM qatracker_testcase \
                  LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_testcase.productid \
                  WHERE qatracker_build.id='"+str(build)+"' AND qatracker_testcase.status='0' \
                  ORDER BY weight ASC, title ASC"

    elif type == "optional":
        sql = "SELECT qatracker_testcase.id, \
                  qatracker_testcase.title, \
                  qatracker_testcase.link \
                  FROM qatracker_testcase \
                  LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_testcase.productid \
                  WHERE qatracker_build.id='"+str(build)+"' AND qatracker_testcase.status='2' \
                  ORDER BY weight ASC, title ASC"

    else:
        sql = "SELECT qatracker_testcase.id, \
                  qatracker_testcase.title, \
                  qatracker_testcase.link \
                  FROM qatracker_testcase \
                  LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_testcase.productid \
                  WHERE qatracker_build.id='"+str(build)+"' AND qatracker_testcase.status != '1' \
                  ORDER BY weight ASC, title ASC"


    cursor.execute(sql)

    testcases = cursor.fetchall()
    return testcases


def getResults(build, testcase, type="all"):

    if type == "fail":
        sql = "SELECT id, status, result FROM qatracker_result \
                WHERE buildid='"+str(build)+"' AND testcaseid='"+str(testcase)+"' AND result = '0' \
                ORDER BY id ASC"
    elif type == "pass":
        sql = "SELECT id, status, result FROM qatracker_result \
               WHERE buildid='"+str(build)+"' AND testcaseid='"+str(testcase)+"' AND result = '1' \
               ORDER BY id ASC"
    else:
        sql = "SELECT id, status, result FROM qatracker_result \
               WHERE buildid='"+str(build)+"' AND testcaseid='"+str(testcase)+"' AND result != '2' \
               ORDER BY id ASC"


    cursor.execute(sql)
    results = cursor.fetchall()
    
    return results

def getBugNumbers(build, testcase, result):

    sql = "SELECT qatracker_bug.bugnumber, max(qatracker_bug.bugimportance) as importance, count(qatracker_bug.bugnumber) as dupe \
            FROM qatracker_bug LEFT JOIN qatracker_result ON qatracker_result.id=qatracker_bug.resultid \
            WHERE qatracker_result.status='0' AND qatracker_result.testcaseid='"+str(testcase)+"' \
            AND qatracker_result.id='"+str(result)+"' AND qatracker_result.buildid='"+str(build)+"'  \
            GROUP BY qatracker_bug.bugnumber ORDER BY importance DESC,dupe DESC,bugnumber ASC"

    cursor.execute(sql)
    bugs = cursor.fetchall()
       
    return bugs

def getBugInformation(bug):

    sql = "SELECT originalbug, title, status, importance FROM qawebsite_launchpad_bug WHERE originalbug='"+str(bug)+"'"

    cursor.execute(sql)
    information = cursor.fetchall()
   
    if information:
        information = information[0]
        infotuple = (information[0], information[1], information[2], information[3])
    else:
        infotuple = (bug, '', '', '')
    return infotuple


def getBugsSummary(milestone):

    images = getImagesForMilestone(milestone)
    
    list_of_bugs = []
    bugs_info = []
    
    for image in images:
        buildid = image[0]
        testcases = getTestcases(buildid, "mandatory")
        for test in testcases:
            testid = test[0]
            results = getResults(buildid, testid)
            for result in results:
                resultid = result[0]
                bugs = getBugNumbers(buildid, testid, resultid)
                for bug in bugs:
                    bugnumber = bug[0]
            
                    if bugnumber not in list_of_bugs:
                        list_of_bugs.append(bugnumber)
                        bugs_info.append(getBugInformation(bugnumber))
   
    #get unfixed bugs
    unfixed = getBugType(bugs_info, "New")
    unfixed += getBugType(bugs_info, "Confirmed")
    unfixed += getBugType(bugs_info, "Triaged")
    unfixed += getBugType(bugs_info, "In Progress")
    unfixed += getBugType(bugs_info, "Incomplete")

    #get fixed bugs
    fixed = getBugType(bugs_info, "Fix Released")
    fixed += getBugType(bugs_info, "Fix Committed")

    #get invalid bugs
    invalid = getBugType(bugs_info, "Invalid")

    report = "\n= Details of bugs =\n"
    report += "\n== Details of Unfixed Issues ==\n"

    for importance in ["Critical", "High", "Medium", "Low", "Undecided", "Undefined"]:
        list = getBugType(unfixed, importance, "importance")
        if list:
            report += "\n=== "+importance+" ===\n"
            for bug in list:
                report += printBug(bug)
        
    report += "\n== Details of Fixed Issues ==\n"

    for importance in ["Critical", "High", "Medium", "Low", "Undecided", "Undefined"]:
        list = getBugType(fixed, importance, "importance")
        if list:
            report += "\n=== "+importance+" ===\n"
            for bug in list:
                report += printBug(bug)
        
    report += "\n== Details of Invalid Issues ==\n\n"

    for bug in invalid:
        report += printBug(bug)
 
    return report

def getFailuresSummary(milestone):

    images = getImagesForMilestone(milestone)
    
    report = "\n\n== Summary of test failures ==\n"

    for image in images:
        buildid = image[0]
        image_name = image[7]
        testcases = getTestcases(buildid, "mandatory")
        for test in testcases:
            testid = test[0]
            test_name = test[1]
            results = getResults(buildid, testid, "fail")
            for result in results:
                resultid = result[0]
                bugs = getBugNumbers(buildid, testid, resultid)
                report += "\n* "+image_name+" - "+test_name+":\n"
                for bug in bugs:
                    bugnumber = bug[0]
                    report += printBug(getBugInformation(bugnumber)) 
   
    return report 

def getGlobalSummary(milestone):

    report = "\n= Summary =\n"
    report += "\n== Test Coverage ==\n"

    # Get mandatory testcases run
    sql = "SELECT count(DISTINCT qatracker_testcase.id) \
            FROM qatracker_build \
            LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid \
            LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid \
            LEFT JOIN qatracker_testcase ON qatracker_testcase.productid = qatracker_product.id \
            LEFT JOIN qatracker_result ON qatracker_result.buildid = qatracker_build.id AND qatracker_result.testcaseid = qatracker_testcase.id \
            WHERE qatracker_milestone.id="+str(milestone)+" AND (qatracker_build.status='0' OR qatracker_build.status='1') AND qatracker_product.siteid='1' \
            AND qatracker_result.result != '2' AND qatracker_testcase.status = '0'"

    cursor.execute(sql)
    mandatory_run = cursor.fetchall()

    mandatory_run = mandatory_run[0][0] if mandatory_run else 0

    # Get total mandatory testcases
    sql = "SELECT count(DISTINCT qatracker_testcase.id) \
            FROM qatracker_build \
            LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid \
            LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid \
            LEFT JOIN qatracker_testcase ON qatracker_testcase.productid = qatracker_product.id \
            WHERE qatracker_milestone.id="+str(milestone)+" AND (qatracker_build.status='0' OR qatracker_build.status='1') AND qatracker_product.siteid='1' \
            AND qatracker_testcase.status = '0'"

    cursor.execute(sql)
    mandatory_total = cursor.fetchall()

    mandatory_total = mandatory_total[0][0] if mandatory_total else 0

    if mandatory_total != 0:
        percentage = (mandatory_run * 100.0)/mandatory_total
        report += "Mandatory Testcase Coverage: %d/%d = %.2f\n" % (mandatory_run, mandatory_total, percentage)

    # Get optional testcases run
    sql = "SELECT count(DISTINCT qatracker_testcase.id) \
            FROM qatracker_build \
            LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid \
            LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid \
            LEFT JOIN qatracker_testcase ON qatracker_testcase.productid = qatracker_product.id \
            LEFT JOIN qatracker_result ON qatracker_result.buildid = qatracker_build.id AND qatracker_result.testcaseid = qatracker_testcase.id \
            WHERE qatracker_milestone.id="+str(milestone)+" AND (qatracker_build.status='0' OR qatracker_build.status='1') AND qatracker_product.siteid='1' \
            AND qatracker_result.result != '2' AND qatracker_testcase.status = '2'"

    cursor.execute(sql)
    optional_run = cursor.fetchall()

    optional_run = optional_run[0][0] if optional_run else 0

    # Get total optional testcases
    sql = "SELECT count(DISTINCT qatracker_testcase.id) \
            FROM qatracker_build \
            LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid \
            LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid \
            LEFT JOIN qatracker_testcase ON qatracker_testcase.productid = qatracker_product.id \
            WHERE qatracker_milestone.id="+str(milestone)+" AND (qatracker_build.status='0' OR qatracker_build.status='1') AND qatracker_product.siteid='1' \
            AND qatracker_testcase.status = '2'"

    cursor.execute(sql)
    optional_total = cursor.fetchall()

    optional_total = optional_total[0][0] if optional_total else 0

    if optional_total != 0:
        percentage = (optional_run * 100.0)/optional_total
        report += "Optional Testcase Coverage: %d/%d = %.2f" % (optional_run, optional_total, percentage)

    # Get total testcases run
    sql = "SELECT count(DISTINCT qatracker_testcase.id) \
            FROM qatracker_build \
            LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid \
            LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid \
            LEFT JOIN qatracker_testcase ON qatracker_testcase.productid = qatracker_product.id \
            LEFT JOIN qatracker_result ON qatracker_result.buildid = qatracker_build.id AND qatracker_result.testcaseid = qatracker_testcase.id \
            WHERE qatracker_milestone.id="+str(milestone)+" AND (qatracker_build.status='0' OR qatracker_build.status='1') AND qatracker_product.siteid='1' \
            AND qatracker_result.result != '2' AND qatracker_testcase.status != '1'"

    cursor.execute(sql)
    total_testcases = cursor.fetchall()

    total_testcases = total_testcases[0][0] if total_testcases else 0

    # Get failures 
    sql = "SELECT count(DISTINCT qatracker_testcase.id) \
            FROM qatracker_build \
            LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid \
            LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid \
            LEFT JOIN qatracker_testcase ON qatracker_testcase.productid = qatracker_product.id \
            LEFT JOIN qatracker_result ON qatracker_result.buildid = qatracker_build.id AND qatracker_result.testcaseid = qatracker_testcase.id \
            WHERE qatracker_milestone.id="+str(milestone)+" AND (qatracker_build.status='0' OR qatracker_build.status='1') AND qatracker_product.siteid='1' \
            AND qatracker_result.result = '0' AND qatracker_testcase.status != '1'"

    cursor.execute(sql)
    total_failures = cursor.fetchall()

    total_failures = total_failures[0][0] if total_failures else 0

    if total_testcases != 0:
        percentage = (total_failures * 100.0)/total_testcases
        report += "\n=== Test Failure Analysis ===\n"
        report += "%d Test Failures\n" % total_failures
        report += "Failure Rate %d/%d = %.2f" % (total_failures, total_testcases, percentage)

    return report 

def getDetailedReport(milestone):

    images = getImagesForMilestone(milestone)
    
   
    report = "\n= Test Coverage Details by Category =\n"

    for image in images:
        bugs_fail = []
        bugs_pass = []
        buildid = image[0]
        image_name = image[7]
        testcases = getTestcases(buildid, "mandatory")
        test_coverage = 0
        report += "\n* "+image_name+":\n"
        for test in testcases:
            test_run = False
            
            testid = test[0]
            test_name = test[1]

            failures = getResults(buildid, testid, "fail")
            if failures:
                test_run = True
                for result in failures:
                    resultid = result[0]
                    bugs = getBugNumbers(buildid, testid, resultid)
                    for bug in bugs:
                        bugnumber = bug[0]
                        if bugnumber not in bugs_fail:
                            bugs_fail.append(bugnumber)
            
            success = getResults(buildid, testid, "pass")
            if success:
                test_run = True
                for result in success:
                    resultid = result[0]
                    bugs = getBugNumbers(buildid, testid, resultid)
                    for bug in bugs:
                        bugnumber = bug[0]
                        if bugnumber not in bugs_fail:
                            bugs_pass.append(bugnumber)
   
            if test_run:
                test_coverage += 1

        report += "\nCoverage: "+str(test_coverage)+"/"+str(len(testcases))+"\n"
        if bugs_fail:
            report += "\nFailures:\n"
            for bug in bugs_fail:
                report += printBug(getBugInformation(bug)) 
    
        if bugs_pass:
            report += "\nBugs (not marked as failures):\n"
            for bug in bugs_pass:
                report += printBug(getBugInformation(bug)) 
    
    return report

def getColumnNumber(sortType):
    
    column = 0

    if sortType == "bug":
        column = 0
    elif sortType == "title":
        column = 1
    elif sortType == "status":
        column = 2
    elif sortType == "importance":
        column = 3
    else:
        column = 0

    return column


def sortBugsBy(buglist, sortType="status"):


    column = getColumnNumber(sortType)
    sortedlist = sorted(buglist, key=operator.itemgetter(column))
   
    return sortedlist


def getBugType(buglist, value, type="status"):

    column = getColumnNumber(type)

    filtered_list = filter((lambda x:(x[column] == value)), buglist)
    
    return filtered_list


def printBug(bug):
    return "LP: #"+str(bug[0])+" - "+bug[1]+" - "+bug[2]+" - "+bug[3]+"\n"



def main():

    usage = "Usage: %prog [options] milestone"

    parser = OptionParser(usage=usage)

    all = True

    parser.add_option('-g', '--global', default=False, action='store_true', dest='global_summary', help='include the global summary')
    parser.add_option('-f', '--failures', default=False, action='store_true', dest='failures', help='include the failures summary')
    parser.add_option('-b', '--bugs', default=False, action='store_true', dest='bugs', help='include the bugs summary')
    parser.add_option('-d', '--detailed', default=False, action='store_true', dest='detailed', help='include the detailed report')
    parser.add_option('-F', '--filename', dest='filename', help='file name to store the report')
    

    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.error("Incorrect number of arguments")
       
    milestone = args[0]

    report = ""

    if options.global_summary:
        all = False
        report += getGlobalSummary(milestone)
    if options.failures:
        all = False
        report += getFailuresSummary(milestone)
    if options.bugs:
        all = False
        report += getBugsSummary(milestone)
    if options.detailed:
        all = False
        report += getDetailedReport(milestone)
  
    if all:
        report += getGlobalSummary(milestone)
        report += getFailuresSummary(milestone)
        report += getBugsSummary(milestone)
        report += getDetailedReport(milestone)
  
    if options.filename:
        f = open(options.filename, 'w')
        f.write(report)
        f.close()
    else:
        print report
    #Close DB connection
    cursor.close()
    db.close()


if __name__ == "__main__":
	main()

