#!/usr/bin/python
# Written by Ara Pulido <ara@ubuntu.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# ATI Product ID
ATI = 98
#nVidia Product ID
NVIDIA = 97 

# Set database settings
# Database settings
db_host="/var/run/postgresql/"
db_user=
db_password=
db_database=

import sys, pgdb, re, os, operator
from optparse import OptionParser

# Connect to the DB
try:
    db=pgdb.connect(user=db_user,password=db_password,host=db_host,database=db_database)
    cursor=db.cursor()
except:
    sys.exit("Failed to connect to the DB")


def getImageForMilestone(milestone, product):
   
    sql = "SELECT qatracker_build.id, qatracker_build.version, \
                qatracker_build.date, qatracker_build.status as buildstatus, \
                qatracker_milestone.title as milestone, qatracker_milestone.status \
                FROM qatracker_build \
                LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid \
                LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid \
                WHERE qatracker_milestone.id='"+str(milestone)+"' AND qatracker_build.status='0' AND qatracker_product.siteid='5' \
                AND qatracker_build.productid = '"+str(product)+"' \
                ORDER BY qatracker_product.title ASC"

    cursor.execute(sql)
    images = cursor.fetchall()

    if images:
        return images[0][0]
    else:
        return images

def getMilestoneName(milestone):
    sql = "SELECT title FROM qatracker_milestone where id = '%s'" % milestone

    cursor.execute(sql)
    name = cursor.fetchall()

    if name:
        return name[0][0]
    else:
        return name

def getUserHW(userid):
    sql = "SELECT value FROM qawebsite_user_setting_info WHERE settingid = '3' AND userid = '%s'" % userid

    cursor.execute(sql)
    
    value = cursor.fetchall()

    if value:
        return value[0][0]
    else:
        return ""

def getUserName(userid):
    sql = "SELECT name FROM users WHERE uid = '%s'" % userid

    cursor.execute(sql)
    
    value = cursor.fetchall()

    if value:
        return value[0][0]
    else:
        return ""

def getTestcases(product, type="all"):

    if type == "mandatory":
        sql = "SELECT qatracker_testcase.id, \
                  qatracker_testcase.title, \
                  qatracker_testcase.link \
                  FROM qatracker_testcase \
                  WHERE qatracker_testcase.productid='"+str(prodcut)+"' AND qatracker_testcase.status='0' \
                  ORDER BY weight ASC, title ASC"

    elif type == "optional":
        sql = "SELECT qatracker_testcase.id, \
                  qatracker_testcase.title, \
                  qatracker_testcase.link \
                  FROM qatracker_testcase \
                  WHERE qatracker_testcase.productid='"+str(product)+"' AND qatracker_testcase.status='2' \
                  ORDER BY weight ASC, title ASC"

    else:
        sql = "SELECT qatracker_testcase.id, \
                  qatracker_testcase.title, \
                  qatracker_testcase.link \
                  FROM qatracker_testcase \
                  WHERE qatracker_testcase.productid='"+str(product)+"' AND qatracker_testcase.status != '1' \
                  ORDER BY weight ASC, title ASC"


    cursor.execute(sql)

    testcases = cursor.fetchall()
    return testcases


def getResults(build, testcase, type="all"):

    if type == "fail":
        sql = "SELECT id, result, reporterid FROM qatracker_result \
                WHERE buildid='"+str(build)+"' AND testcaseid='"+str(testcase)+"' AND result = '0' \
                ORDER BY id ASC"
    elif type == "pass":
        sql = "SELECT id, result, reporterid FROM qatracker_result \
               WHERE buildid='"+str(build)+"' AND testcaseid='"+str(testcase)+"' AND result = '1' \
               ORDER BY id ASC"
    else:
        sql = "SELECT id, result, reporterid FROM qatracker_result \
               WHERE buildid='"+str(build)+"' AND testcaseid='"+str(testcase)+"' AND result != '2' \
               ORDER BY id ASC"


    cursor.execute(sql)
    results = cursor.fetchall()
    
    return results

def getBugNumbers(build, testcase, result):

    sql = "SELECT qatracker_bug.bugnumber, max(qatracker_bug.bugimportance) as importance, count(qatracker_bug.bugnumber) as dupe \
            FROM qatracker_bug LEFT JOIN qatracker_result ON qatracker_result.id=qatracker_bug.resultid \
            WHERE qatracker_result.status='0' AND qatracker_result.testcaseid='"+str(testcase)+"' \
            AND qatracker_result.id='"+str(result)+"' AND qatracker_result.buildid='"+str(build)+"'  \
            GROUP BY qatracker_bug.bugnumber ORDER BY importance DESC,dupe DESC,bugnumber ASC"

    cursor.execute(sql)
    bugs = cursor.fetchall()
       
    return bugs

def getBugInformation(bug):

    sql = "SELECT originalbug, title, status, importance FROM qawebsite_launchpad_bug WHERE originalbug='"+str(bug)+"'"

    cursor.execute(sql)
    information = cursor.fetchall()
   
    if information:
        information = information[0]
        infotuple = (information[0], information[1], information[2], information[3])
    else:
        infotuple = (bug, '', '', '')
    return infotuple


def getColumnNumber(sortType):
    
    column = 0

    if sortType == "bug":
        column = 0
    elif sortType == "title":
        column = 1
    elif sortType == "status":
        column = 2
    elif sortType == "importance":
        column = 3
    else:
        column = 0

    return column


def sortBugsBy(buglist, sortType="status"):


    column = getColumnNumber(sortType)
    sortedlist = sorted(buglist, key=operator.itemgetter(column))
   
    return sortedlist


def getBugType(buglist, value, type="status"):

    column = getColumnNumber(type)

    filtered_list = filter((lambda x:(x[column] == value)), buglist)
    
    return filtered_list


def printBug(bug):
    return "LP: #"+str(bug[0])+" - "+bug[1]+" - "+bug[2]+" - "+bug[3]+"\n"

def appendTemplate(template, report=""):
    f = open(template)
    report += f.read()
    f.close()
    return report

def addImage(image_name, report):

    report += "\n<h1>%s</h1>\n" % image_name

    return report

def addTableHeaders(testcases, report):
    report += '\n<table class="xorg-table">\n<thead>\n<tr>\n'

    report += "<th>&nbsp;</th>\n"
    for test in testcases:
        report += "<th>%s</th>\n" % test

    report += "</tr>\n<thead>\n"

    return report

def addCloseTable(report):

    report += "\n</tbody>\n</table>\n"

    return report

def addSummary(milestone_summary, report):
    
    report += "<tbody>\n<tr>\n"

    for value in milestone_summary:
        report += "<td>%s</td>\n" % value 

    report += "</tr>\n"
    
    return report

def closeReport(report):
    
    report += "</body>\n</html>"

    return report


def main():


    usage = "Usage: %prog [options] milestone"

    parser = OptionParser(usage=usage)

    all = True

    parser.add_option('-F', '--filename', dest='filename', help='file name to store the report') 

    (options, args) = parser.parse_args()

    if len(args) < 1:
        parser.error("Incorrect number of arguments")
       
    milestones = args

    report = ""
    report = appendTemplate("template.html")
    
    images = ["ATI", "NVIDIA"]

    for image in images:
        report = addImage(image, report)

        product = ATI if image == "ATI" else NVIDIA

        testcases = getTestcases(product)

        testcases_name = map(operator.itemgetter(1), testcases)
        report = addTableHeaders(testcases_name, report)

        for milestone in milestones:

            milestone_name = getMilestoneName(milestone)

            image = getImageForMilestone(milestone, product)

            image_summary = [milestone_name]
            for test in testcases:
                testid = test[0]
                results = getResults(image, testid)
                failures = getResults(image, testid, "fail")

                summary = "%d/%d" % (len(failures), len(results))
                if len(results) == 0:
                    image_summary.append(summary)
                else:
                    image_summary.append(linkDetail(milestone, testid, summary))

                detail_header = "<h1>%s - %s</h1>" % (milestone_name, test[1])
                report_detail = appendTemplate("template.html")
                report_detail += detail_header
                report_detail = appendTemplate("template_detail.html", report_detail) 
                for result in results:
                    resultid = result[0]
                    pass_fail = "<td class=result-pass>Pass</td>" if result[1] == 1 else "<td class=result-fail>Fail</td>"
                    testerid = result[2]

                    bugs = map(operator.itemgetter(0), getBugNumbers(image, testid, resultid))

                    model = getUserHW(testerid)
                    tester = getUserName(testerid)
                    report_detail += "<tr>"
                    report_detail += "\n<td>%s</td>\n" % tester
                    report_detail += "\n<td>%s</td>\n" % model
                    report_detail += pass_fail
                    report_detail += "\n<td>"
                    for bug in bugs:
                        report_detail += linkBug(bug) + " "
                    report_detail += "</td>\n"
                    report_detail += "</tr>"
                
                report_detail = addCloseTable(report_detail)

                report_detail = closeReport(report_detail)

                filename = "%s_%s.html" % (milestone, testid)
                f = open(filename, "w")
                f.write(report_detail)
                f.close()

            report = addSummary(image_summary, report)

        report = addCloseTable(report)
        
        report = closeReport(report)

        f = open("xorg_report.html", "w")
        f.write(report)
        f.close()


    if options.filename:
        f = open(options.filename, 'w')
        f.write(report)
        f.close()
    else:
        print report
    #Close DB connection
    cursor.close()
    db.close()


def linkBug(bug):
    return '<a href="http://bugs.launchpad.net/bugs/%d">%d</a>' % (bug, bug)

def linkDetail(milestoneid, testcaseid, summary):
    return '<a href="%s_%s.html">%s</a>' % (milestoneid, testcaseid, summary)

if __name__ == "__main__":
    main()
   
       
       


